# coding=utf-8
import uuid

class Order(object):
    def __init__(self, account_id: str, symbol: str, volume: int, order_type: int, order_biz:int, price:float=0, sid:str = '', comment:str='') -> None:
        """
        报单委托
        :param account_id:              账户
        :param symbol:                  委托标的
        :param volume:                  委托量
        :param order_type:              委托类型
        :param order_biz:               委托业务类型
        :param price:                   委托价格
        :param sid:                     用户自定义的委托ID，唯一值，不能出现重复。不填写则自动生成一个 uuid
        :param comment:                 备注说明
        """
        self.account_id = account_id
        self.symbol = symbol
        self.volume = volume
        self.order_type = order_type
        self.order_biz = order_biz
        self.price = price
        self.sid = sid if sid!='' else uuid.uuid4().hex
        self.comment = comment

class CancelOrder(object):
    def __init__(self, sid: str, comment: str='') -> None:
        """
        撤单委托
        :param sid:                     报单时的sid
        :param comment:                 备注
        """
        self.sid = sid
        self.comment = comment

class RawFunc(object):
    def __init__(self, account_id: str, func_id: str, func_params: str, rid: str='', comment: str='') -> None:
        """
        功能号委托对象
        :param  rid:                     用户策略定义的功能号请求ID, 唯一性, 不可重复使用 
        :param  account_id:             掘金交易账号ID
        :param  func_id:                功能号ID
        :param  func_params:            功能号参数，json字符串
        :param  comment:                备注说明, 不需要时可给空字符串.          
        """
        self.account_id = account_id
        self.func_id = func_id
        self.func_params = func_params
        self.rid = rid if rid !='' else uuid.uuid4().hex
        self.comment = comment
        