# coding=utf-8

import random
from typing import List
from common import CancelOrder, Order, RawFunc
import csv
import os
from datetime import datetime
import pandas as pd

headers_order = ['sid', 'account_id', 'symbol', 'volume', 'order_type', 'order_business(order_biz)', 'price', 'comment']
headers_cancel_order = ['sid', 'comment']
headers_raw_func = ['rid', 'account_id', 'func_id', 'func_params', 'comment']

table_place_order = None
table_cancel_order = None
table_raw_func = None


def _generate_csv(folder: str, typ: str) -> str:
    return os.path.join(folder,"{}.{}.csv".format(datetime.now().strftime('%Y%m%d%H%M%S.%f'), typ))


def place_order(csv_dir: str, order: Order) -> str:
    """下一个单"""
    return place_orders(csv_dir, [order])[0]


def place_orders(csv_dir: str, orders: List[Order]) -> List[str]:
    """下多个单在一个文件"""
    global table_place_order
    rows = []
    sids = []
    for order in orders:
        rows.append(
            (order.sid, order.account_id, order.symbol, order.volume, order.order_type, order.order_biz, order.price, order.comment)
        )
        sids.append(order.sid)

    if table_place_order is None:
        csv_file = _generate_csv(csv_dir, 'order')
        # 注意：这里需要设置 newline='', 否则csv header 与记录之间会出现一个空行。详见：https://stackoverflow.com/questions/3348460/csv-file-written-with-python-has-blank-lines-between-each-row
        table_place_order = open(csv_file, 'w', newline='')
        w = csv.writer(table_place_order)
        w.writerow(headers_order)
    else:
        w = csv.writer(table_place_order)

    w.writerows(rows)
    table_place_order.flush()

    return sids
    

def cancel_order(csv_dir: str, order: CancelOrder) -> str:
    """撤单"""
    return cancel_orders(csv_dir, [order])[0]


def cancel_orders(csv_dir: str, orders: List[CancelOrder]) -> List[str]:
    """一个文件撤多个单"""
    global table_cancel_order
    sids = []
    rows = []
    if table_cancel_order is None:
        csv_file = _generate_csv(csv_dir, 'cancel_order')
        # 注意：这里需要设置 newline='', 否则csv header 与记录之间会出现一个空行。详见：https://stackoverflow.com/questions/3348460/csv-file-written-with-python-has-blank-lines-between-each-row
        table_cancel_order = open(csv_file, 'w', newline='')
        w = csv.writer(table_cancel_order)
        w.writerow(headers_cancel_order)
    else:
        w = csv.writer(table_cancel_order)
    for order in orders:
        rows.append(
            (order.sid, order.comment)
        )
        sids.append(order.sid)

    w.writerows(rows)
    table_cancel_order.flush()

    return sids


def raw_func(csv_dir: str, order: RawFunc) -> str:
    """下功能号单"""
    return raw_funcs(csv_dir, [order])[0]


def raw_funcs(csv_dir: str, orders: List[RawFunc]) -> List[str]:
    """下多个功能号单在一个文件"""
    global table_raw_func
    rids = []
    rows = []
    if table_cancel_order is None:
        csv_file = _generate_csv(csv_dir, 'func_request')
        # 注意：这里需要设置 newline='', 否则csv header 与记录之间会出现一个空行。详见：https://stackoverflow.com/questions/3348460/csv-file-written-with-python-has-blank-lines-between-each-row
        table_raw_func = open(csv_file, 'w', newline='')
        w = csv.writer(table_raw_func)
        w.writerow(headers_raw_func)
    else:
        w = csv.writer(table_raw_func)

    for order in orders:
        rows.append(
            (order.rid, order.account_id, order.func_id, order.func_params, order.comment)
        )
        rids.append(order.rid)

    w.writerows(rows)
    table_raw_func.flush()

    return rids


def getCash(account_id):
    """获取资金"""
    return pd.read_csv(f'{out_path}\{account_id}\cash.csv')


def getPosition(account_id):
    """获取持仓"""
    return pd.read_csv(f'{out_path}\{account_id}\position.csv')


def getExecrpt(account_id):
    """获取成交"""
    return pd.read_csv(f'{out_path}\{account_id}\execution_report.csv')


def getOrder(account_id):
    """获取委托"""
    return pd.read_csv(f'{out_path}\{account_id}\order_status.csv')


def getFunc(account_id):
    """获取功能号"""
    return pd.read_csv(f'{out_path}\{account_id}\\func_response.csv')


if __name__ == '__main__':
    '''
    【温馨提示】当前示例程序仅供参考，请勿用于实盘交易
    '''
    symbols = [
        "SHSE.600000", "SHSE.600004", "SHSE.600006", "SHSE.600007", "SHSE.600008", "SHSE.600009", "SHSE.600010",
        "SHSE.600012", "SHSE.600015", "SHSE.600016", "SHSE.600017", "SHSE.600018", "SHSE.600019", "SHSE.600020",
        "SZSE.000001", "SZSE.000002", "SZSE.000023", "SZSE.000035", "SZSE.000058", "SZSE.000088", "SZSE.000517",
        "SZSE.300001", "SZSE.300024", "SZSE.300051", "SZSE.300062", "SZSE.300076", "SZSE.300094", "SZSE.300117"
    ]

    # 文件单结果输出路径：登录掘金3终端 -> 手工交易 -> 文件单 -> 文件单输入 -> 新增扫单的扫单路径 
    out_path = r'D:\workspace\be\sanner-strategy\examples\out'
    
    # 文件单结果输入路径：登录掘金3终端 -> 手工交易 -> 文件单 -> 文件单输出-> 输出路径
    in_path = r'D:\workspace\be\sanner-strategy\examples\in'

    # 仿真/实盘的账户ID
    account_id = '0a0d76c0-32ba-11eb-9210-001018b67b94'

    order = Order(
            account_id=account_id,
            symbol=random.choice(symbols),
            volume=100,
            order_type=2,
            order_biz=1,
            price=0,
            comment='我的委托'
        )

    for i in range(2):
        # sid = place_order(csv_dir=in_path, order=order)
        sid = cancel_order(csv_dir=in_path, order=CancelOrder('123'))
        print(sid, order.symbol)
    # print(getCash(account_id))
    # print(getPosition(account_id))
    # print(getExecrpt(account_id))
    # print(getOrder(account_id))
    # print(getFunc(account_id))
