# coding=utf-8

import os
import random
import csv
from typing import List
from datetime import datetime, timedelta
import random
import json
import time
import logging
from common import CancelOrder, Order, RawFunc, ready
from scan_by_csv import *
logging.basicConfig(level = logging.DEBUG, format = '%(message)s', filename='log.csv', filemode='w')
POINTER_POSITION = 0


def get_reported_orders(account_id: str, sids: List[str]) -> List:
    global POINTER_POSITION
    endTime = {}
    find = False
    csv_file = os.path.join(out_path, f'{account_id}/order_status_change.csv')
    f = open(csv_file, encoding='utf-8-sig')
    while not find:
        # print('找了一次')
        time.sleep(0.001)
        data = []
        for sid in sids:
            f.seek(POINTER_POSITION)  # 移动文件指针到之前读过的位置。
            while True:
                record = f.readline()
                if record == '':
                    break
                record = record.split(',')
                if record[1] == sid and record[8] == '1':  # 1是sid， 8是status sid在寻找的sid中，且status是已报。
                    data.append(record)
                    endTime[sid] = time.time()
                    break
        if len(data) == len(sids):
            find = True

    POINTER_POSITION = f.tell()
    f.close()
    return endTime


def count_spend_time(end_time: datetime, start_time: datetime) -> int:
    '''
    计算耗时，返回毫秒
    '''
    delta = end_time - start_time
    return int(delta.seconds * 1000 + delta.microseconds / 1000)


def perf_order(place_total=100, place_interval=2):
    '''
    逐笔委托性能测试
    :param  place_total:         总计下多少笔委托
    :param  place_interval:      每笔委托报单间隔(秒)
    '''
    if place_total <= 0:
        return
    total_spend = 0
    for i in range(place_total):
        start = time.time()
        order = Order(
            account_id= account_id,
            symbol= random.choice(symbols),
            volume=100,
            order_type=2,
            order_biz=1,
            price=0,
            comment='我的,委托',
        )
        sid = place_order(in_path, order)
        # print("\n", sid)
        endTime = get_reported_orders(account_id, [sid])
        for sid in endTime:
            logging.debug(f'{sid},{start},{endTime[sid]},{endTime[sid] - start}')

        # d = {"index": i+1, "start": f"{start}", "end": f"{end}", "spend": spend}
        # print(json.dumps(d))
        total_spend += (endTime[sid] - start)
        time.sleep(place_interval)
    # print(f"\n逐笔委托 {place_total} 笔，每笔委托间隔 {place_interval} 秒，平均每笔委托从报单到查询到委托已报耗时 {total_spend / place_total} 毫秒")


def perf_orders(place_total: int=100):
    start = datetime.now()
    print("开始委托：", start)
    orders=[]
    for i in range(place_total):
        order = Order(
            account_id= account_id,
            symbol= random.choice(symbols),
            volume=100,
            order_type=2,
            order_biz=1,
            price=0,
            comment='我的委托',
        )
        orders.append(order)

    start = time.time()

    sids = place_orders(in_path, orders)
    # time.sleep(1)
    # 循环判断所有单都已报
    endTime = get_reported_orders(account_id, sids)
    for sid in endTime:
        logging.debug(f'{sid},{start},{endTime[sid]},{endTime[sid] - start}')
    
    # end = datetime.now()
    # spend = count_spend_time(end, start)
    # print(f"\n开始时间：{start}，全部已报：{end}，批量委托 {place_total} 笔，总耗时：{spend} 毫秒")


def set_pointer_position(account_id: str):
    """
    设置委托流水文件 order_status_change.csv 每次开始扫描的起始位置，避免每次启动扫单策略都要从第一行开始扫描。
    这个函数要在发起委托前调用。
    """
    global POINTER_POSITION
    csv_file = os.path.join(out_path, f'{account_id}/order_status_change.csv')
    if os.path.exists(csv_file):
        f = open(csv_file, encoding='utf-8-sig')
        f.seek(0, 2) # 跳转到文件末尾
        POINTER_POSITION = f.tell()
        f.close()


if __name__ == "__main__":
    # 标的，随机下单
    symbols = [
        "SHSE.600000", "SHSE.600004", "SHSE.600006", "SHSE.600007", "SHSE.600008", "SHSE.600009", "SHSE.600010",
        "SHSE.600012", "SHSE.600015", "SHSE.600016", "SHSE.600017", "SHSE.600018", "SHSE.600019", "SHSE.600020",
        "SZSE.000001", "SZSE.000002", "SZSE.000023", "SZSE.000035", "SZSE.000058", "SZSE.000088", "SZSE.000517",
        "SZSE.300001", "SZSE.300024", "SZSE.300051", "SZSE.300062", "SZSE.300076", "SZSE.300094", "SZSE.300117"
    ]
    
    # 文件单结果输出路径：登录掘金3终端 -> 手工交易 -> 文件单 -> 文件单输入 -> 新增扫单的扫单路径 
    out_path = r'D:\COMPANY\扫单优化及历史交易数据管理\out'
    
    # 文件单结果输入路径：登录掘金3终端 -> 手工交易 -> 文件单 -> 文件单输出-> 输出路径
    in_path = r'D:\COMPANY\扫单优化及历史交易数据管理\in'

    # 仿真/实盘的账户ID
    account_id = '56fc6ea9-362d-11eb-bda5-001018b67b94'

    # 在发起委托前调用，用于设置委托流水文件每次的起始扫描位置
    set_pointer_position(account_id=account_id)

    # 逐笔委托
    # for i in range(3000):
    perf_order(1000, 0.1)

    # 批量委托
    # perf_orders(3000)
