# coding=utf-8

import os
import random
from typing import List

import dbf
from datetime import datetime, timedelta
import random
from common import Order
from scan_by_dbf import finish_place_order, place_order, place_orders
import json
import time
import csv

headers = ['sid', 'start_time', 'end_time', 'spend']

def get_reported_orders(account_id: str, sid: List[str], place_time: datetime)->List:
    dbf_file = os.path.join(out_path, f'{account_id}/order_status_change.dbf')
    table = dbf.Table(dbf_file, codepage='utf8')
    table.open()
    fields = table.field_names
    data = []
    length = len(table)
    for idx in range(length-1, -1, -1):
        d = {}
        for field in fields:
            val = table[idx][field]
            if isinstance(val, str):
                val=str(val).strip()
            d[field]=val
        if d.get('SID') in sid and d.get('STATUS') == 1:
            data.append(d)

        # UTC 转 东八区时间
        created_at = datetime.strptime(d.get('CREATED_AT'), '%Y-%m-%dT%H:%M:%S.%fZ') + timedelta(hours=8)
        if created_at < place_time:
            # print("break")
            # 调出循环
            break

    table.close()
    return data

def count_spend_time(end_time: datetime, start_time: datetime)->int:
    '''
    计算耗时，返回毫秒
    '''
    delta = end_time - start_time
    return delta.seconds * 1000 + delta.microseconds / 1000

def perf_order(place_total=100, place_interval=2)->List:
    '''
    逐笔委托性能测试
    :param  place_total:         总计下多少笔委托
    :param  place_interval:      每笔委托报单间隔(秒)
    '''
    if place_total <= 0:
        return
    total_spend = 0
    rows = []
    for i in range(place_total):
        start = datetime.now()
        order = Order(
            account_id= account_id,
            symbol= random.choice(symbols),
            volume=100,
            order_type=2,
            order_biz=1,
            price=0,
            comment='我的,委托',
        )
        sid = place_order(in_path, order)
        print("\n", sid, order.symbol)
        find = False
        while not find:
            time.sleep(0.001)
            data = get_reported_orders(account_id, [sid], start)
            if len(data)>0:
                find=True

        end = datetime.now()
        spend = count_spend_time(end, start)
        d={"index": i+1, "start": f"{start}", "end": f"{end}", "spend": spend}
        rows.append([sid, start, end, spend])
        print(json.dumps(d))
        total_spend += spend
        time.sleep(place_interval)
    print(f"\n逐笔委托 {place_total} 笔，每笔委托间隔 {place_interval} 秒，平均每笔委托从报单到查询到委托已报耗时 {total_spend / place_total} 毫秒")
    return rows


def perf_orders(place_total: int=100)->List:
    start = datetime.now()
    print("开始委托：", start)
    orders=[]
    for i in range(place_total):
        order = Order(
            account_id= account_id,
            symbol= random.choice(symbols),
            volume=100,
            order_type=2,
            order_biz=1,
            price=0,
            comment='我的委托',
        )
        orders.append(order)

    start = datetime.now()

    sids = place_orders(in_path, orders)
    found_sids = []
    rows = []
    while len(found_sids) !=place_total:
        time.sleep(0.001)
        found = get_reported_orders(account_id, sids, start)
        for item in found:
            end = datetime.now()
            sid = item.get('SID')
            rows.append([sid, start, end, count_spend_time(end, start)])
            found_sids.append(sid)
            sids.remove(sid)

    end = datetime.now()
    spend = count_spend_time(end, start)
    print(f"\n开始时间：{start}，全部已报：{end}，批量委托 {place_total} 笔，总耗时：{spend} 毫秒")
    return rows


def save_perf_res(filename: str, rows: List):
    with open(filename, 'w', newline='') as f:
        writer = csv.writer(f)
        writer.writerow(headers)
        writer.writerows(rows)


if __name__=="__main__":
    # 标的，随机下单
    symbols = [
        "SHSE.600000", "SHSE.600004", "SHSE.600006", "SHSE.600007", "SHSE.600008", "SHSE.600009", "SHSE.600010",
        "SHSE.600012", "SHSE.600015", "SHSE.600016", "SHSE.600017", "SHSE.600018", "SHSE.600019", "SHSE.600020",
        "SHSE.600022", "SHSE.600023", "SHSE.600025", "SHSE.600026", "SHSE.600027", "SHSE.600028", "SHSE.600029",
        "SHSE.600031", "SHSE.600033", "SHSE.600035", "SHSE.600036", "SHSE.600037", "SHSE.600038", "SHSE.600039",
        "SHSE.600050", "SHSE.600051", "SHSE.600052", "SHSE.600053", "SHSE.600054", "SHSE.600055", "SHSE.600056",
        "SHSE.600058", "SHSE.600059", "SHSE.600060", "SHSE.600061", "SHSE.600062", "SHSE.600063", "SHSE.600064",
        "SHSE.600067", "SHSE.600068", "SHSE.600069", "SHSE.600070", "SHSE.600071", "SHSE.600072", "SHSE.600073",
        "SHSE.600076", "SHSE.600077", "SHSE.600078", "SHSE.600079", "SHSE.600080", "SHSE.600081", "SHSE.600082",
        "SHSE.600084", "SHSE.600085", "SHSE.600086", "SHSE.600088", "SHSE.600089", "SHSE.600090", "SHSE.600093",
        "SHSE.600095", "SHSE.600096", "SHSE.600097", "SHSE.600098", "SHSE.600099", "SHSE.600100", "SHSE.600101",
        "SHSE.600104", "SHSE.600105", "SHSE.600106", "SHSE.600107", "SHSE.600108", "SHSE.600109", "SHSE.600110",
        "SHSE.600112", "SHSE.600113", "SHSE.600114", "SHSE.600115", "SHSE.600116", "SHSE.600117", "SHSE.600118",
        "SHSE.600120", "SHSE.600121", "SHSE.600122", "SHSE.600123", "SHSE.600125", "SHSE.600126", "SHSE.600127",
        "SHSE.600129", "SHSE.600130", "SHSE.600131", "SHSE.600132", "SHSE.600133", "SHSE.600135", "SHSE.600136",
    ]
    
    # 文件单结果输出路径：登录掘金3终端 -> 手工交易 -> 文件单 -> 文件单输入 -> 新增扫单的扫单路径 
    out_path = r'D:\workspace\be\sanner-strategy\examples\out'
    
    # 文件单结果输入路径：登录掘金3终端 -> 手工交易 -> 文件单 -> 文件单输出-> 输出路径
    in_path = r'D:\workspace\be\sanner-strategy\examples\in'

    # 仿真/实盘的账户ID
    account_id = '0a0d76c0-32ba-11eb-9210-001018b67b94'

    # 逐笔委托
    num = 1000
    perf_res_filename = f'sence3-1个文件{num}个委托每个间隔10ms.csv'
    rows = []
    for i in range(num):
        rows = rows + perf_order(1, 0.01)
        # finish_place_order()
    save_perf_res(perf_res_filename, rows)

    # 批量委托
    # perf_orders(num)
